## what are the rows of the variables of interest, that is Linf (each cohort), k(each cohort), t0 on the natural scale

require(MASS)

VB.f = function (Linf,k,t0,Age) {
    
        Length_age = Linf*(1-exp(-k*(Age-t0)))
            
    return(Length_age)
}


predictNLS <- function(
object, 
newdata, 
nsim = 1000,
...
)
{
  require(MASS, quietly = TRUE)
   
  ## get right-hand side of formula
  RHS <- as.list(object$call$formula)[[3]]
  EXPR <- as.expression(RHS)
   
  ## all variables in model
  VARS <- all.vars(EXPR)
   
  ## coefficients
  COEF <- coef(object)
   
  ## extract predictor variable    
  predNAME <- setdiff(VARS, names(COEF))  
   
  ## take fitted values, if 'newdata' is missing
  if (missing(newdata)) {
    newdata <- eval(object$data)[predNAME]
    colnames(newdata) <- predNAME
  }
     
  ## check that 'newdata' has same name as predVAR
  if (names(newdata)[1] != predNAME) stop("newdata should have name '", predNAME, "'!")
   
  ## get parameter coefficients
  COEF <- coef(object)
     
  ## get variance-covariance matrix
  VCOV <- vcov(object)
   
  ## augment variance-covariance matrix for 'mvrnorm' 
  ## by adding a column/row for 'error in x'
  NCOL <- ncol(VCOV)
  ADD1 <- c(rep(0, NCOL))
  ADD1 <- matrix(ADD1, ncol = 1)
  colnames(ADD1) <- predNAME
  VCOV <- cbind(VCOV, ADD1)
  ADD2 <- c(rep(0, NCOL + 1))
  ADD2 <- matrix(ADD2, nrow = 1)
  rownames(ADD2) <- predNAME
  VCOV <- rbind(VCOV, ADD2) 
         
  ## iterate over all entries in 'newdata' as in usual 'predict.' functions
  NR <- nrow(newdata)
  respVEC <- numeric(NR)
  seVEC <- numeric(NR)
  varPLACE <- ncol(VCOV)   
   
  ## define counter function
  counter <- function (i) 
  {
    if (i%%10 == 0) 
      cat(i)
    else cat(".")
    if (i%%50 == 0) 
      cat("\n")
    flush.console()
  }
   
  outMATnls <- NULL 
   
  for (i in 1:NR) {
    counter(i)
     
    ## get predictor values and optional errors
    predVAL <- newdata[i, 1]
    if (ncol(newdata) == 2) predERROR <- newdata[i, 2] else predERROR <- 0
    names(predVAL) <- predNAME  
    names(predERROR) <- predNAME  
     
    ## create mean vector for 'mvrnorm'
    MU <- c(COEF, predVAL)
     
    ## create variance-covariance matrix for 'mvrnorm'
    ## by putting error^2 in lower-right position of VCOV
    newVCOV <- VCOV
    newVCOV[varPLACE, varPLACE] <- predERROR^2
     
    ## create MC simulation matrix
    simMAT <- mvrnorm(n = nsim, mu = MU, Sigma = newVCOV, empirical = TRUE)
     
    ## evaluate expression on rows of simMAT
    EVAL <- try(eval(EXPR, envir = as.data.frame(simMAT)), silent = TRUE)
    if (inherits(EVAL, "try-error")) stop("There was an error evaluating the simulations!")
     
    ## collect statistics
    PRED <- data.frame(predVAL)
    colnames(PRED) <- predNAME   
    FITTED <- predict(object, newdata = data.frame(PRED))
    MEAN.sim <- mean(EVAL, na.rm = TRUE)
    SD.sim <- sd(EVAL, na.rm = TRUE)
    MEDIAN.sim <- median(EVAL, na.rm = TRUE)
    MAD.sim <- mad(EVAL, na.rm = TRUE)
    QUANT <- quantile(EVAL, probs = c(0.025,0.975))
    RES <- c(as.numeric(predVAL),FITTED, MEAN.sim, SD.sim, MEDIAN.sim, MAD.sim, QUANT[1], QUANT[2])
    outMATnls <- rbind(outMATnls, RES)
  }
   
  colnames(outMATnls) <- c("Age","Fitted","Mean","Sd","Median","Mad","Quant_low","Quant_high")
  rownames(outMATnls) <- NULL
   
  cat("\n")
   
  return(as.data.frame(outMATnls))  
  
  }
  
  
  
  
  
  
  
  
  
  
  
nsim = 10000

size.label.x = 25
size.text.x = 15

size.label.y = 25
size.text.y = 15
size.legend.text = 15
size.legend.title = 15
unit.legend.h = 1.4
unit.legend.w = 1.4
size.point = 3
line.lwd = 2

zak.std = read.table("m_grow3_zak_Linfcohort_kcohort.std",header = T)

table.std = as.data.frame(with(zak.std, table(name)))

zak.fit =  read.admbFit('/Users/mrag/Dropbox/Simone\ and\ Hans/m_grow3_zak_Linfcohort_kcohort')

Linf_sd.vett = which(zak.std$name == "Linf_sd")
k_sd.vett = which(zak.std$name == "k_sd")
t0_sd.vett = which(zak.std$name =="beta_t0")

index.keep = c(t0_sd.vett,Linf_sd.vett,k_sd.vett)

zak.V = matrix(rep(0,length(index.keep)^2),length(index.keep))

for (i in 1:length(index.keep)) {

for (j in 1:length(index.keep)) {

zak.V[i,j] = zak.fit$cov[index.keep[i],index.keep[j]]

}

}


row.names(zak.V) = c("t0","Linf_98","Linf_99","Linf_00","Linf_01","Linf_02","Linf_03",
"Linf_04","Linf_05","Linf_06","Linf_07","Linf_08","k_98","k_99","k_00","k_01","k_02","k_03",
"k_04","k_05","k_06","k_07","k_08")

colnames(zak.V) = c("t0","Linf_98","Linf_99","Linf_00","Linf_01","Linf_02","Linf_03",
"Linf_04","Linf_05","Linf_06","Linf_07","Linf_08","k_98","k_99","k_00","k_01","k_02","k_03",
"k_04","k_05","k_06","k_07","k_08")


#### which cohort we want to keep





Linf.keep = which(colnames(zak.V)=="Linf_99") 

k.keep = which(colnames(zak.V)=="k_99")

t0.keep = which(colnames(zak.V)=="t0") 

coh.keep = c(t0.keep, Linf.keep, k.keep)

zak.coh.V = matrix(rep(0,length(coh.keep)^2),length(coh.keep))

for (i in 1:length(coh.keep)) {

for (j in 1:length(coh.keep)) {

zak.coh.V[i,j] = zak.V[coh.keep[i],coh.keep[j]]

}

}


data= m_grow3_zak_Linfcohort_kcohort.par

data$beta[2:length(data$beta)]	= data$beta[2:length(data$beta)] + data$beta[1]

data$beta = exp(data$beta*10)

data$beta_k0[2:length(data$beta_k0)]	= data$beta_k0[2:length(data$beta_k0)] + data$beta_k0[1]

data$beta_k0 = exp(data$beta_k0)


cohort.to.keep = 2

COEF = matrix(0,1,3)

colnames(COEF) = c("t0","Linf","k")

COEF[1] = data$beta_t0
COEF[2] = data$beta[2]
COEF[3] = data$beta_k0[2]


predNAME = "Age"

newdata = matrix(1:9,ncol=1)

colnames(newdata) = predNAME

VCOV = zak.coh.V

## augment variance-covariance matrix for 'mvrnorm' 
## by adding a column/row for 'error in x'
  NCOL <- ncol(VCOV)
  ADD1 <- c(rep(0, NCOL))
  ADD1 <- matrix(ADD1, ncol = 1)
  colnames(ADD1) <- predNAME
  VCOV <- cbind(VCOV, ADD1)
  ADD2 <- c(rep(0, NCOL + 1))
  ADD2 <- matrix(ADD2, nrow = 1)
  rownames(ADD2) <- predNAME
  VCOV <- rbind(VCOV, ADD2) 


NR <- nrow(newdata)
  respVEC <- numeric(NR)
  seVEC <- numeric(NR)
  varPLACE <- ncol(VCOV)   
   
  ## define counter function
  counter <- function (i) 
  {
    if (i%%10 == 0) 
      cat(i)
    else cat(".")
    if (i%%50 == 0) 
      cat("\n")
    flush.console()
  }
   
  outMAT <- NULL 


for (i in 1:NR) {
    counter(i)
     
    ## get predictor values and optional errors
    predVAL <- newdata[i, 1]
    if (ncol(newdata) == 2) predERROR <- newdata[i, 2] else predERROR <- 0
    names(predVAL) <- predNAME  
    names(predERROR) <- predNAME  
     
    ## create mean vector for 'mvrnorm'
    MU <- c(COEF, predVAL)
     
    ## create variance-covariance matrix for 'mvrnorm'
    ## by putting error^2 in lower-right position of VCOV
    newVCOV <- VCOV
    newVCOV[varPLACE, varPLACE] <- predERROR^2
     
    ## create MC simulation matrix
    simMAT <- mvrnorm(n = nsim, mu = MU, Sigma = newVCOV, empirical = TRUE)
     
    ## evaluate expression on rows of simMAT
     #VB.f = function (Linf,k,t0,Age)
     
     EVAL = rep(0,nsim)
     for (jj in 1:nsim) {
     
     EVAL[jj] = VB.f(simMAT[jj,2],simMAT[jj,3],simMAT[jj,1],as.numeric(simMAT[i,4])) # col 4 is age
     
     }
     
    ## collect statistics
    PRED <- data.frame(predVAL)
    colnames(PRED) <- predNAME   
    FITTED <- VB.f(COEF[2],COEF[3],COEF[1],as.numeric(simMAT[i,4]))
    AGE = as.numeric(simMAT[i,4])
    MEAN.sim <- mean(EVAL, na.rm = TRUE)
    SD.sim <- sd(EVAL, na.rm = TRUE)
    MEDIAN.sim <- median(EVAL, na.rm = TRUE)
    MAD.sim <- mad(EVAL, na.rm = TRUE)
    QUANT <- quantile(EVAL, probs = c(0.025,0.975))
    RES <- c(AGE,FITTED, MEAN.sim, SD.sim, MEDIAN.sim, MAD.sim, as.numeric(QUANT[1]), as.numeric(QUANT[2]))
    outMAT <- rbind(outMAT, RES)
  }
  
  colnames(outMAT) = c("Age","Fitted","Mean","Sd","Median","Mad","Quant_low","Quant_high")
  row.names(outMAT) = NULL
  outMAT = data.frame(outMAT)
  
  
  
data.df = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11")

data.df$Cohort = factor(data.df$Cohort)

dddata = subset(data.df,Cohort=="C99")

  
  fit.nls = nls(Length ~ Linf.nls*(1-exp(-k.nls*(Age-t0.nls))),data = dddata,
start = list(Linf.nls = 300, k.nls = 0.5, t0.nls = 0.3))

predNAME = "Age"

newdata = data.frame(Age=1:9)

nls.predict.df = predictNLS(fit.nls,newdata = newdata,nsim = nsim)

  

  
  require(ggplot2)
  require(grid)
  
    ggbands <- ggplot() 
    
  ggbands<- ggbands  + 
  #geom_point(data = dddata, aes(x = Age, y = Length),size = 3, colour ="gray50") +
  geom_line(data = outMAT,aes(x = Age, y = Fitted),col="black",lwd = line.lwd) +
  geom_line(data = nls.predict.df,aes(x = Age, y = Fitted),col="black",lwd = line.lwd,linetype = 2) +
  geom_line(data = dddata,aes(x = Age, y = Length,group = Mark),colour="gray50",lwd = 0.8,alpha = 0.2) +
      geom_ribbon(data=outMAT,aes(x = Age,ymin=Quant_low,ymax=Quant_high),alpha = 0.3) + 
      geom_ribbon(data=nls.predict.df,aes(x = Age,ymin=Quant_low,ymax=Quant_high),alpha = 0.3) +
      theme(plot.title = element_text(lineheight=.8, face="bold",size = 25),
    plot.background = element_blank()
   ,panel.grid.major = element_blank()
   ,panel.grid.minor = element_blank()
   ,panel.border = element_blank()
   ,panel.background = element_blank()) +theme(axis.line = element_line(color = 'black')) +
      scale_y_continuous(breaks=seq(100,max(dddata$Length+30),50),limits=c(100,max(dddata$Length+30))) +
	  scale_x_continuous(breaks=seq(1,max(dddata$Age+1),1),limits=c(1, max(dddata$Age+1))) +
	   theme(plot.margin = unit(c(2,1,2,1), "cm"),
	   axis.title.x = element_text(size=size.label.x,vjust=-2),
           axis.text.x  = element_text(size=size.text.x),
           axis.title.y = element_text(size=size.label.y, vjust = 0.1),
           axis.text.y  = element_text(size=size.text.y)) +
           labs(x="Age", y ="Length")
           
           ggbands
    
    