### create and plot mock data set #######

require(grid)
require(ggplot2)

size.label.x = 25
size.text.x = 20
line.lwd = 1.2
size.label.y = 25
size.text.y = 20




VBsnov3 = function (beta,k0,log_sigma_v,v,log_sigma_u,u,t0,log_sigma_w,w,plpo,maxage=10) {
    age = seq(0,maxage,1)
    Length_age = rep(0,length(age))
    gamma = exp(beta*10 + exp(log_sigma_v)*v)
    k = exp(k0) * exp( exp(log_sigma_u)*u )
    Linf = gamma 
    t0 = t0 + exp(log_sigma_w)*w
    
    for (ii in 1:length(age)) {
        
        Length_age[ii] = Linf*(1-exp(-k*(age[ii]-t0)))
        
    }
    if (plpo==0) {
        plot(Length_age ~ age, xlim = c(0,max(age)),ylim = c(0,max(Length_age)),pch=16)
    }
    
    else if (plpo==1) {
        points(Length_age ~ age,pch=15)
    }
    
    return(data.frame(Age=age,Length=Length_age))
}



set.seed(9)
maxage = 7

for (i in 1:3) {
#generate a mock file of 200 individuals of age 1 to 10 using 3 random effects uncorrelated
####values are similar to those found for the population of Zakojska

n.tot.ind = 200

beta = 0.58
beta_k0 = -1
beta_t0 = -0.38
log_sigma_v = -1.5
log_sigma_u = -1.5
log_sigma_w = -3

mockrand = data.frame(Mark = (1:n.tot.ind),u = rep(0,n.tot.ind),w = rep(0,n.tot.ind),v=rep(0,n.tot.ind))


uv = NormCorr.f(n.tot.ind,rho = -0.9)
mockrand$u = uv[,1]
mockrand$v = uv[,2]
mockrand$w = rnorm(n.tot.ind,0,0)

uv.real.cor = cor.test(mockrand$u,mockrand$v)


mockdataset = data.frame(Mark = rep(mockrand$Mark,each=maxage+1),Length = rep(0,n.tot.ind*(maxage+1)),Age = rep(0,n.tot.ind*(maxage+1)))
contlen = 1
for (i in 1:n.tot.ind) {
plot.mock =VBsnov3(beta,beta_k0,log_sigma_v,mockrand[i,"v"],log_sigma_u,
mockrand[i,"u"],beta_t0, log_sigma_w,mockrand[i,"w"], 2,maxage=maxage)

mockdataset$Length[contlen: ((maxage+1)*i)] = plot.mock$Length
mockdataset$Age[contlen: ((maxage+1)*i)] = plot.mock$Age


contlen = length(mockdataset$Length[mockdataset$Length>0])+1
}


}




gmock <- ggplot(subset(mockdataset,Mark >30 & Mark<=50), aes(x = Age, y = Length)) 

#g <- ggplot(subset(uppidri.df,Mark!="NA" & Month == 9), aes(x = Age, y = Weight)) 

gmock.negcorr <- gmock + geom_line(aes(group = Mark),col="black",lwd = line.lwd) +
      #geom_smooth(aes(group = 1), colour = 'red', size = 1) +
  #     ggtitle("Simulated data") +
      theme(plot.title = element_text(lineheight=.8, face="bold",size = 25),
    plot.background = element_blank()
   ,panel.grid.major = element_blank()
   ,panel.grid.minor = element_blank()
   ,panel.border = element_blank()
   ,panel.background = element_blank()) +theme(axis.line = element_line(color = 'black')) +
      scale_y_continuous(breaks=c(0,50,100,150,200,250,300,350,400,450),limits=c(0,450)) +
	  scale_x_continuous(breaks=c(0,1,2,3,4,5,6,7),limits=c(0, 7.5)) +
	   theme(plot.margin = unit(c(2,1,2,1), "cm"),
	   axis.title.x = element_text(size=size.label.x,vjust=-2),
           axis.text.x  = element_text(size=size.text.x),
           axis.title.y = element_text(size=size.label.y, vjust = 0.1),
           axis.text.y  = element_text(size=size.text.y)) +
           labs(x="Age", y ="Length")
	  

set.seed(7)

for (i in 1:3) {
#generate a mock file of 200 individuals of age 1 to 10 using 3 random effects uncorrelated
####values are similar to those found for the population of Zakojska

n.tot.ind = 200

beta = 0.58
beta_k0 = -1
beta_t0 = -0.38
log_sigma_v = -1.5
log_sigma_u = -1.5
log_sigma_w = -3

mockrand = data.frame(Mark = (1:n.tot.ind),u = rep(0,n.tot.ind),w = rep(0,n.tot.ind),v=rep(0,n.tot.ind))


uv = NormCorr.f(n.tot.ind,rho = 0.6)
mockrand$u = uv[,1]
mockrand$v = uv[,2]
mockrand$w = rnorm(n.tot.ind,0,0)

uv.real.cor = cor.test(mockrand$u,mockrand$v)


mockdataset = data.frame(Mark = rep(mockrand$Mark,each=maxage+1),Length = rep(0,n.tot.ind*(maxage+1)),Age = rep(0,n.tot.ind*(maxage+1)))
contlen = 1
for (i in 1:n.tot.ind) {
plot.mock =VBsnov3(beta,beta_k0,log_sigma_v,mockrand[i,"v"],log_sigma_u,
mockrand[i,"u"],beta_t0, log_sigma_w,mockrand[i,"w"], 2,maxage=maxage)

mockdataset$Length[contlen: ((maxage+1)*i)] = plot.mock$Length
mockdataset$Age[contlen: ((maxage+1)*i)] = plot.mock$Age


contlen = length(mockdataset$Length[mockdataset$Length>0])+1
}

}

gmock <- ggplot(subset(mockdataset,Mark >29 & Mark<=49), aes(x = Age, y = Length)) 

#g <- ggplot(subset(uppidri.df,Mark!="NA" & Month == 9), aes(x = Age, y = Weight)) 

gmock.poscorr <- gmock + geom_line(aes(group = Mark),col="black",lwd = line.lwd) +
      #geom_smooth(aes(group = 1), colour = 'red', size = 1) +
  #     ggtitle("Simulated data") +
      theme(plot.title = element_text(lineheight=.8, face="bold",size = 25),
    plot.background = element_blank()
   ,panel.grid.major = element_blank()
   ,panel.grid.minor = element_blank()
   ,panel.border = element_blank()
   ,panel.background = element_blank()) +theme(axis.line = element_line(color = 'black')) +
      scale_y_continuous(breaks=c(0,50,100,150,200,250,300,350,400,450),limits=c(0,450)) +
	  scale_x_continuous(breaks=c(0,1,2,3,4,5,6,7),limits=c(0, 7.5)) +
	   theme(plot.margin = unit(c(2,1,2,1), "cm"),
	   axis.title.x = element_text(size=size.label.x,vjust=-2),
           axis.text.x  = element_text(size=size.text.x),
           axis.title.y = element_text(size=size.label.y, vjust = 0.1),
           axis.text.y  = element_text(size=size.text.y)) +
           labs(x="Age", y ="")
	  
	  
	
	
set.seed(7)

for (i in 1:3) {
#generate a mock file of 200 individuals of age 1 to 10 using 3 random effects uncorrelated
####values are similar to those found for the population of Zakojska

n.tot.ind = 200

beta = 0.58
beta_k0 = -1
beta_t0 = -0.38
log_sigma_v = -1.5
log_sigma_u = -1.5
log_sigma_w = -3

mockrand = data.frame(Mark = (1:n.tot.ind),u = rep(0,n.tot.ind),w = rep(0,n.tot.ind),v=rep(0,n.tot.ind))


uv = NormCorr.f(n.tot.ind,rho = 0)
mockrand$u = uv[,1]
mockrand$v = uv[,2]
mockrand$w = rnorm(n.tot.ind,0,0)

uv.real.cor = cor.test(mockrand$u,mockrand$v)


mockdataset = data.frame(Mark = rep(mockrand$Mark,each=maxage+1),Length = rep(0,n.tot.ind*(maxage+1)),Age = rep(0,n.tot.ind*(maxage+1)))
contlen = 1
for (i in 1:n.tot.ind) {
plot.mock =VBsnov3(beta,beta_k0,log_sigma_v,mockrand[i,"v"],log_sigma_u,
mockrand[i,"u"],beta_t0, log_sigma_w,mockrand[i,"w"], 2,maxage=maxage)

mockdataset$Length[contlen: ((maxage+1)*i)] = plot.mock$Length
mockdataset$Age[contlen: ((maxage+1)*i)] = plot.mock$Age


contlen = length(mockdataset$Length[mockdataset$Length>0])+1
}

}

gmock <- ggplot(subset(mockdataset,Mark >29 & Mark<=49), aes(x = Age, y = Length)) 

#g <- ggplot(subset(uppidri.df,Mark!="NA" & Month == 9), aes(x = Age, y = Weight)) 

gmock.nocorr <- gmock + geom_line(aes(group = Mark),col="black",lwd = line.lwd) +
      #geom_smooth(aes(group = 1), colour = 'red', size = 1) +
  #     ggtitle("Simulated data") +
      theme(plot.title = element_text(lineheight=.8, face="bold",size = 25),
    plot.background = element_blank()
   ,panel.grid.major = element_blank()
   ,panel.grid.minor = element_blank()
   ,panel.border = element_blank()
   ,panel.background = element_blank()) +theme(axis.line = element_line(color = 'black')) +
      scale_y_continuous(breaks=c(0,50,100,150,200,250,300,350,400,450),limits=c(0,450)) +
	  scale_x_continuous(breaks=c(0,1,2,3,4,5,6,7),limits=c(0, 7.5)) +
	   theme(plot.margin = unit(c(2,1,2,1), "cm"),
	   axis.title.x = element_text(size=size.label.x,vjust=-2),
           axis.text.x  = element_text(size=size.text.x),
           axis.title.y = element_text(size=size.label.y, vjust = 0.1),
           axis.text.y  = element_text(size=size.text.y)) +
           labs(x="Age", y ="")
	  	
	
arrange_ggplot2(gmock.negcorr,gmock.poscorr,gmock.nocorr,ncol=3) 
	  
	  
	  
	  
