####All Analyses

### First, we source a function taken shamelessly from this webpage of Michigan State University 
# http://qfc.fw.msu.edu/userfun.asp that recovers .par and .cor output of admb 
#taked from 

source("read.admbFit.r")


# Then, we load some libraries (install them if you don't have them already installed) 
require(ggplot2)
require(grid)
require(MASS)
require(R2admb)
require(MCMCglmm)


# Unzip or unrar the compressed file EBgrowth in the directory in which you will work with R 


###  Import raw data in R ######

allstreams.gatsnik.df = readRDS('gatsnik.EB.df.RDS')
allstreams.zakojska.df = readRDS('zakojska.EB.df.RDS')

# the data for both populations stops in 2012 and both datasets also include the parental cohort 
#                      Metadata
# Mark = ID of fish (it is the number on a Carlin tag)
# Length = total length of fish (mm)
# Weight = weight of fish (g)
# Age = age of fish (fish are 0 in the year they are born, 1 after the first winter and so on)
# Year = year when the fish at that age was sampled
# Cohort = year of birth cohort (e.g. C98 = cohort born in 1998, C01 = cohort born in 2001)
# Density_first_year = fish density (n/ha) during the fish first year of life


## In model fitting, I divided Density_first_year by 1000 to get better convergence

#########  mean length and t-tests for length at age for Gacnik and Zakojska ####

####### mean length at age for gacnik and zakojska, we exclude for both populations the stocking 
# cohort (P) and the Cohort 2011 (it does not give any contribution, since they have been observed
# in 2012 for the first time
gat.lmean1 = with(subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==1],na.rm = T))
zak.lmean1 = with(subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==1],na.rm = T))

gat.lmean2 = with(subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==2],na.rm = T))
zak.lmean2 = with(subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==2],na.rm = T))

gat.lmean3 = with(subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==3],na.rm = T))
zak.lmean3 =with(subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==3],na.rm = T))

gat.lmean4 = with(subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==4],na.rm = T))
zak.lmean4 = with(subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==4],na.rm = T))

gat.lmean5 = with(subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==5],na.rm = T))
zak.lmean5 = with(subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==5],na.rm = T))

gat.lmean6 = with(subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==6],na.rm = T))
zak.lmean6 = with(subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==6],na.rm = T))

gat.lmean7 = with(subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==7],na.rm = T))
zak.lmean7 = with(subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==7],na.rm = T))

gat.lmean8 = with(subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==8],na.rm = T))
zak.lmean8 = with(subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==8],na.rm = T))

gat.lmean9 = with(subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==9],na.rm = T))
zak.lmean9 = with(subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"),
mean(Length[Age==9],na.rm = T))


##### data of length at age for gacnik and zakojska
gat.l1 = subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11" & Age == 1)$Length
zak.l1 = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11" & Age == 1)$Length

gat.l2 = subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11" & Age == 2)$Length
zak.l2 = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"  & Age == 2)$Length

gat.l3 = subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11" & Age == 3)$Length
zak.l3 = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11" & Age == 3)$Length

gat.l4 = subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11" & Age == 4)$Length
zak.l4 = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11"  & Age == 4)$Length

gat.l5 = subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11" & Age == 5)$Length
zak.l5 = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11" & Age == 5)$Length

gat.l6 = subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11" & Age == 6)$Length
zak.l6 = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11" & Age == 6)$Length

gat.l7 = subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11" & Age == 7)$Length
zak.l7 = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11" & Age == 7)$Length

gat.l8 = subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11" & Age == 8)$Length
zak.l8 = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11" & Age == 9)$Length

gat.l9 = subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11" & Age == 9)$Length
zak.l9 = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11" & Age == 9)$Length

gat.l10 = subset(allstreams.gatsnik.df, Cohort!="P" & Cohort!="C11" & Age == 10)$Length
zak.l10 = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11" & Age == 10)$Length


#### t-test for differences in mean length at age in gacnik and zakojska

t.test(gat.l1,zak.l1)
t.test(gat.l2,zak.l2)
t.test(gat.l3,zak.l3)
t.test(gat.l4,zak.l4)
t.test(gat.l5,zak.l5)
t.test(gat.l6,zak.l6)
t.test(gat.l7,zak.l7)
t.test(gat.l8,zak.l8)
t.test(gat.l9,zak.l9)
t.test(gat.l10,zak.l10)



########


###### let's run the models listed in Table 1 in the manuscript #####
### in order to run the models for Gacnik you need at least 16 GB of RAM (otherwise the computer
# frezees), for Zakojsa 4 GB are enough (see specifications in the README.txt file)
#If you want to run just the model for Zakojska (or Gacnik) just create a new file with extension .r
# (from gatzak model chain.r) with just the part for Zakojska (or Gacnik) and source it. 

#### I worked only with a MAC, but everything should be doable also on Windows and Linux
### first thing you should compile the .tpl file in the same directory in which you are working 
# with R
## if you use a MAC, just (1) install admb (http://admb-project.org/). (2) you should have ADMB in 
#the path. (3) In the shell, move with cd to the directory where you are 
#working in R (which is the same where you unzipped EBgrowth), and compile with m_grow3.tpl
# with        admb -r m_grow3
# you should ge the message "Done" (it means there were no errors when compiling) 

#If you use Windows please have a look here
## https://github.com/jlaake/marked/blob/master/marked/inst/README.txt

# the command below estimates all models included in gatzak model chain.r, they are the same 
# presented in Table 1 in the manuscript
# when estimating (i.e. fitting) models, a file (different for each model) is sourced
# e.g. m_grow3_gat_Linfdensity.r. That file has the instructions for creating the model matrix and 
# creating a .dat file that will be used by admb to get the input information

source("gatzak model chain.r")


#### if you don't want to run the models, here are parameters (.par) and predicted length(s)
## (.rep, hindcasting)

##For Gacnik
#model with no predictors
m_grow3_gat0_noP.par = readRDS("m_grow3_gat0_noP.par.RDS")
m_grow3_gat0_noP.rep = readRDS("m_grow3_gat0_noP.rep.RDS")

 #model with density as predictors for both Linf and k
m_grow3_gat_Linfdensity.par = readRDS("m_grow3_gat_Linfdensity.par.RDS")
m_grow3_gat_Linfdensity.rep = readRDS("m_grow3_gat_Linfdensity.rep.RDS")

#model with density as predictor Linf 
#and cohort for k
m_grow3_gat_Linfdensity_kcohort.par = readRDS("m_grow3_gat_Linfdensity_kcohort.par.RDS")
m_grow3_gat_Linfdensity_kcohort.rep = readRDS("m_grow3_gat_Linfdensity_kcohort.rep.RDS")

#model with cohort as predictor for k
m_grow3_gat_kcohort.par = readRDS("m_grow3_gat_kcohort.par.RDS")
m_grow3_gat_kcohort.rep = readRDS("m_grow3_gat_kcohort.rep.RDS")

#model with density as predictor for k
m_grow3_gat_kdensity.par = readRDS("m_grow3_gat_kdensity.par.RDS")
m_grow3_gat_kdensity.rep = readRDS("m_grow3_gat_kdensity.rep.RDS")

#model with cohort as predictor Linf 
#and density for k
m_grow3_gat_Linfcohort_kdensity.par = readRDS("m_grow3_gat_Linfcohort_kdensity.par.RDS")
m_grow3_gat_Linfcohort_kdensity.rep = readRDS("m_grow3_gat_Linfcohort_kdensity.rep.RDS")

#model with cohort as predictor for Linf
m_grow3_gat_Linfcohort.par = readRDS("m_grow3_gat_Linfcohort.par.RDS")
m_grow3_gat_Linfcohort.rep = readRDS("m_grow3_gat_Linfcohort.rep.RDS")

#model with cohort as predictor for Linf and k
m_grow3_gat_Linfcohort_kcohort.par = readRDS("m_grow3_gat_Linfcohort_kcohort.par.RDS")
m_grow3_gat_Linfcohort_kcohort.rep = readRDS("m_grow3_gat_Linfcohort_kcohort.rep.RDS")

#model with density as predictor for Linf and k
m_grow3_gat_Linfdensity_kdensity.par = readRDS("m_grow3_gat_Linfdensity_kdensity.par.RDS")
m_grow3_gat_Linfdensity_kdensity.rep = readRDS("m_grow3_gat_Linfdensity_kdensity.rep.RDS")

# For Zakojska
#model with no predictors
m_grow3_zak0_noP.par = readRDS("m_grow3_zak0_noP.par.RDS")
m_grow3_zak0_noP.rep = readRDS("m_grow3_zak0_noP.rep.RDS")

 #model with density as predictors for both Linf and k
m_grow3_zak_Linfdensity.par = readRDS("m_grow3_zak_Linfdensity.par.RDS")
m_grow3_zak_Linfdensity.rep = readRDS("m_grow3_zak_Linfdensity.rep.RDS")

#model with density as predictor Linf 
#and cohort for k
m_grow3_zak_Linfdensity_kcohort.par = readRDS("m_grow3_zak_Linfdensity_kcohort.par.RDS")
m_grow3_zak_Linfdensity_kcohort.rep = readRDS("m_grow3_zak_Linfdensity_kcohort.rep.RDS")

#model with cohort as predictor for k
m_grow3_zak_kcohort.par = readRDS("m_grow3_zak_kcohort.par.RDS")
m_grow3_zak_kcohort.rep = readRDS("m_grow3_zak_kcohort.rep.RDS")


#model with density as predictor for k
m_grow3_zak_kdensity.par = readRDS("m_grow3_zak_kdensity.par.RDS")
m_grow3_zak_kdensity.rep = readRDS("m_grow3_zak_kdensity.rep.RDS")

#model with cohort as predictor Linf 
#and density for k
m_grow3_zak_Linfcohort_kdensity.par = readRDS("m_grow3_zak_Linfcohort_kdensity.par.RDS")
m_grow3_zak_Linfcohort_kdensity.rep = readRDS("m_grow3_zak_Linfcohort_kdensity.rep.RDS")

#model with cohort as predictor for Linf
m_grow3_zak_Linfcohort.par = readRDS("m_grow3_zak_Linfcohort.par.RDS")
m_grow3_zak_Linfcohort.rep = readRDS("m_grow3_zak_Linfcohort.rep.RDS")

#model with cohort as predictor for Linf and k
m_grow3_zak_Linfcohort_kcohort.par = readRDS("m_grow3_zak_Linfcohort_kcohort.par.RDS")
m_grow3_zak_Linfcohort_kcohort.rep = readRDS("m_grow3_zak_Linfcohort_kcohort.rep.RDS")

#model with density as predictor for Linf and k
m_grow3_zak_Linfdensity_kdensity.par = readRDS("m_grow3_zak_Linfdensity_kdensity.par.RDS")
m_grow3_zak_Linfdensity_kdensity.rep = readRDS("m_grow3_zak_Linfdensity_kdensity.rep.RDS")




######################## test the correlation between k and Linf at the individuals level 
##with the model with no predictors


source("Correlation between linf and k.r")

cor.k.linf.zak.only1 #corr for zakojska,fish sampled just once

cor.k.linf.zak.over2 #corr for zakojska,fish sampled more than 2 times

cor.k.linf.gat.only1 #corr for  gacnik,fish sampled more than 2 times

cor.k.linf.gat.over2 #corr for gacnik,fish sampled just once



######################### plot data predicted by model against data observed 
##### (hindcasting) for Gacnik
##left plot is for the model with no predictors, right plot for the model with cohort as 
# predictor for both k and link

source("Plot gat hindcasting.r")
#### R squared and mean absolute error for model with no predictors
Rsq.gat0
mean.err.pred.gat0

##### R squared and mean absolute error for model with cohort as predictor for k and Linf
Rsq.gat.Link.cohort
mean.err.pred.gat.Link.cohort

##########################plot data predicted by model against data observed 
##(hindcasting) for Zakojska
##left plot is for the model with no predictors, right plot for the model with cohort as 
# predictor for both k and link

source("Plot zak hindcasting.r")
#### R squared and mean absolute error for model with no predictors
Rsq.zak0
mean.err.pred.zak0

##### R squared and mean absolute error for model with cohort as predictor for k and Linf
Rsq.zak.Link.cohort
mean.err.pred.zak.Link.cohort

########################### validation for Zakojska (as explained in the manuscript)

#### read into the workspace the parameters of the validation model with cohort
# as predictor for both k and linf . This way
## I do not have to rerun the model every time. 71 refers to seed, therefore it is fully
## reproducible. For how the validation 
#is carried out, please refer to the manuscript
results.par.Linf.c.k.c.zak71 = readRDS("results.par.Linf.c.k.c.zak71.RDS")

##you can also rerun the same mode by sourcing 
#  validation_zak_Link_k_cohort.r and at the top of it 
# instead of seed.series = c(41:61), use seed.series = c(71:71)

### this produces 2 figures, on the left the validation on the model with cohort as
# predictor for both k and linf and on the right the validation for the model using mean
#length at age of the cohort of the fish
# the 

source("Plot validation zak #71.r")



###### print r squared and mean absolute error for the model with cohort
# as predictor for both k and linf
Rsq.pred 
mean.err.pred


###### print r squared and mean absolute error for the model using mean
#length at age of the cohort of the fish
Rsq.avg
mean.err.avg

############ validation for Gacnik

#### get into the workspace the parameters of the validation model with cohort
# as predictor for both k and linf . This way
## I do not have to rerun the model every time. 42 refers to seed. For how the validation 
#is carried out, please refer to the manuscript
results.par.Linf.c.k.c.gat42 = readRDS("results.par.Linf.c.k.c.gat42.RDS")

##you can also rerun the same mode by sourcing 
#  validation_gat_Link_k_cohort.r and at the top of it 
# instead of seed.series = c(41:61), use seed.series = c(42:42)

### this produces 2 figures, on the left the validation on the model with cohort as
# predictor for both k and linf and on the right the validation for the model using mean
#length at age of the cohort of the fish
# the 

source("Plot validation gat #42.r")

###### print r squared and mean absolute error for the model with cohort
# as predictor for both k and linf
Rsq.pred 
mean.err.pred


###### print r squared and mean absolute error for the model using mean
#length at age of the cohort of the fish
Rsq.avg
mean.err.avg


######## Here below is the code for mutiple runs of validation for Gacnik and Zakojska,
### for the models with cohort as predictor for both k and linf and with no
##  predictors

### source("validation multigatzak.r") # not run it takes a day or so

## I used seed from 41 to 61, 20 validation samples for each model, for a total of 80
# validations considering both populations

##Read the R squared and mean absolute error for each model and each population

## pred = model predictions (cohort as predictor),
# avg = predictions with mean length at age of cohort

Rsq.pred.vect.Linf.k.cohort.zak4161 = readRDS("Rsq.pred.vect.Linf.k.cohort.zak4161")
Rsq.avg.vect.Linf.k.cohort.zak4161 = readRDS("Rsq.avg.vect.Linf.k.cohort.zak4161")
MAE.pred.vect.Linf.k.cohort.zak4161 = readRDS("MAE.pred.vect.Linf.k.cohort.zak4161")
MAE.avg.vect.Linf.k.cohort.zak4161 = readRDS("MAE.avg.vect.Linf.k.cohort.zak4161")

Rsq.pred.vect.Linf.k.cohort.gat4161 = readRDS("Rsq.pred.vect.Linf.k.cohort.gat4161")
Rsq.avg.vect.Linf.k.cohort.gat4161 = readRDS("Rsq.avg.vect.Linf.k.cohort.gat4161")
MAE.pred.vect.Linf.k.cohort.gat4161 = readRDS("MAE.pred.vect.Linf.k.cohort.gat4161")
MAE.avg.vect.Linf.k.cohort.gat4161 = readRDS("MAE.avg.vect.Linf.k.cohort.gat4161")


## pred = model predictions (no predictor), 
## avg = predictions with mean length at age of whole population


Rsq.pred.vect.zak04161 = readRDS("Rsq.pred.vect.zak04161")
Rsq.avg.vect.zak04161 = readRDS("Rsq.avg.vect.zak04161")
MAE.pred.vect.zak04161 = readRDS("MAE.pred.vect.zak04161")
MAE.avg.vect.zak04161 = readRDS("MAE.avg.vect.zak04161")


Rsq.pred.vect.gat04161 = readRDS("Rsq.pred.vect.gat04161")
Rsq.avg.vect.gat04161 = readRDS("Rsq.avg.vect.gat04161")
MAE.pred.vect.gat04161 = readRDS("MAE.pred.vect.gat04161")
MAE.avg.vect.gat04161 = readRDS("MAE.avg.vect.gat04161")


##### get mean estimate and standard errors of the parameters of the model with cohort as
# predictor for both k and linf and with no  predictors

## file needed are m_grow3_zak_Linfcohort_kcohort.std , m_grow3_zak_noP.std,
# m_grow3_gat_Linfcohort_kcohort.std , m_grow3_gat_noP.std 
# when standard errors of parameters are calculated
# admb uses the delta method to estimate standard error of parameter p when 
# p is function of g (actual parameter estimated by the model)

source("zakojska std.r")
source("gatsnik std.r")

## if you want to re-estimate standar errors, just run the model from R using
### system("./m_grow3"), then grab the .std file

## table with mean estimate and 95% confidence interval
## (min,max) for model with cohort as predictor 
## mean and conf int refer to fixed parameters (no random effects), so we have a different
## k and linf for each cohort (from the oldest -top- to youngest -bottom)
zak_Linfcohort_kcohort.err
gat_Linfcohort_kcohort.err

# table with mean estimate and 95% confidence interval
## (min,max) for model with no predictors 
## mean and conf int refer to fixed parameters (no random effects), and we have just one
## estimate (whole population), since we do not have any predictor.
gat0.err
zak0.err


## repeatability ##

source("repeatability.r")

#### mode of the posterior (postmode) and 95% posterior credible intervals (hdp)
# all fish
gatrep.postmode
gatrep.hpd
zakrep.postmode
zakrep.hpd

## only fish younger than 4 years old

gatrep.red.postmode
gatrep.red.hpd
zakrep.red.postmode
zakrep.red.hdp


#### plot density plot of random effects and plot k-linf at the individual level
### top row zakojska, bottom row gacnik

## with no predictors
source("Plot histo mixed effects gat zak ggplot2.r")

## with cohort as predictor for both k and linf
source("Plot histo mixed effects gat zak with coh effect ggplot2.r")


#### simulate growth trajectories with negatively correlated (left)
## positively correlated (middle)
## and no correlation (right) k and lin at the individual level

source("Correlation linf k.r")


### plot mean growth for cohorts with the random-effect model and nls model
### zakojska cohort 2001 left and 1999 right
source("Plot zak cohorts.r")

## gacnik cohort 2001 left and 2002 right

source("Plot gat cohorts.r")



##### Empirical growth trajectories and frequency of number of time fish were sampled
### top row Gacnik (truncated at age 10)
#### bottom row Zakojska 
source("Empirical growth traj and samples.r")



####  This routine is for testing whether the fitting random-effect algorithm is
## artificially creating a correlation (or viceversa) between k and linf when there
## is no correlation (or viceversa) in the starting data

## In the 10 plots (you can change them by simply changing the seed in the
## sourced code. Random numbers are used for the real correlation and for "thinning" the 
## dataset, i.e. some observations are deleted in order to create a realistic dataset)
## rr refers to real correlation, re to estimated correlation (i.e. 
## individual-level parameters are estimated using the empirical bayes method, then the 
## Pearson's r is estimated). In addition, in this example there is variation at the 
### individual level in t0, but t0 is estimated as fixed (without random effects), using
## standard deviation of the distribution of individual effects in t0 similar to those 
## estimated for k (u) and linf (v) for Gacnik and Zakojska

source("Plot test correlation linf k.r")


# Plot of growth trajectories

source("Plot cohort traj.r")

# Correlation among Linf and k at the individual level within cohorts

# Zakojska
source("Cor k Linf cohort zak.r")

### print the series of Pearson's r for the various cohorts (starting with the oldest)

cor.series.zak

# Gacnik
source("Cor k Linf cohort gat.r")

### print the series of Pearson's r for the various cohorts (starting with the oldest)

cor.series.gat

### Plot confidence bands using MonteCarlo simulations (see supplementary material Fig A3)
# this does not work in a shell because ggplot2 alpha transparency is not supported

source("ConfBands.r")

### Checking whether the correlation between estimates at the cohort level may be artifact

####  This routine is for testing whether the fitting random-effect algorithm is
## artificially creating a correlation (or viceversa) between k and linf at the cohort level when there
## is no correlation (or viceversa) in the starting data

# be patient! 
source("Plot test correlation linf k_cohort.r")

#If you don't to run it, just load the results (they are the same as in Table A3)
read.csv("cor.test.coh.csv",header = T)


## If you want to test JAGS for fitting the random-effects VBGMs

source("vb.jags.on.r")

## if you want to test the function nlme in the package nlme of R open the file 
# "VBGF nlme.r". It can be sourced, but it contains both the analyses for simulated data
# and for the population of Zakojska. Specifications are inside.