require(ggplot2)
require(grid)

## Function for arranging ggplots. use png(); arrange(p1, p2, ncol=1); dev.off() to save.
vp.layout <- function(x, y) viewport(layout.pos.row=x, layout.pos.col=y)
arrange_ggplot2 <- function(..., nrow=NULL, ncol=NULL, as.table=FALSE) {
	dots <- list(...)
	n <- length(dots)
	if(is.null(nrow) & is.null(ncol)) { nrow = floor(n/2) ; ncol = ceiling(n/nrow)}
	if(is.null(nrow)) { nrow = ceiling(n/ncol)}
	if(is.null(ncol)) { ncol = ceiling(n/nrow)}
        ## NOTE see n2mfrow in grDevices for possible alternative
grid.newpage()
pushViewport(viewport(layout=grid.layout(nrow,ncol) ) )
	ii.p <- 1
	for(ii.row in seq(1, nrow)){
	ii.table.row <- ii.row	
	if(as.table) {ii.table.row <- nrow - ii.table.row + 1}
		for(ii.col in seq(1, ncol)){
			ii.table <- ii.p
			if(ii.p > n) break
			print(dots[[ii.table]], vp=vp.layout(ii.table.row, ii.col))
			ii.p <- ii.p + 1
		}
	}
}


results.par = m_grow3_zak0_noP.par  # get model parameters (model with no predictors)

pred = m_grow3_zak0_noP.rep  #get predicted data

data.df = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11") #data

#create a dataframe with observed data (Obs) and predicted by model 
obpred = data.frame(Obs = data.df$Length, Pred = pred) 


### compute Sum of Squared Errors
SSE.zak0 = sum((obpred$Obs-obpred$Pred)^2)
### compute Total Sum of Squares
SST.zak0 = sum((obpred$Obs-mean(obpred$Obs))^2)
#compute R squared
Rsq.zak0 = 1-(SSE.zak0/SST.zak0)

#compute mean absolute error
mean.err.pred.zak0 = sqrt(mean((obpred$Obs-obpred$Pred)^2))




#seed 42
# in RStudio I use as #size plot Width = 1000, Height = 500

#I don't like to hardcode variables, from this panel I can modify size of 
#plot attributes
size.label.x = 25
size.text.x = 20
size.point = 5
size.label.y = 25
size.text.y = 20
size.legend.text = 15
size.legend.title = 15
unit.legend.h = 1.4
unit.legend.w = 1.4


#create plot with ggplot2

ggzak.mc <- ggplot(subset(obpred), aes(x = Pred, y = Obs)) 		  
ggazak.mc1  <- ggzak.mc + geom_point(pch = 16, size = 3, colour = "gray50") +
coord_fixed() +
geom_abline(intercept=0, slope=1, linetype = 2, lwd=1.2) +
 #      ggtitle("mediantimesnik") +
      theme(plot.title = element_text(lineheight=.8, face="bold"), 
      plot.background = element_blank()
   ,panel.grid.major = element_blank()
   ,panel.grid.minor = element_blank()
   ,panel.border = element_blank()
   ,panel.background = element_blank(),
   axis.line = element_line(color = 'black'),
   plot.margin = unit(c(1.5,1.5,1.5,1.5), "cm"),
   axis.title.x = element_text(size=size.label.x,vjust=-2),
           axis.text.x  = element_text(size=size.text.x ),
           axis.title.y = element_text(size=size.label.y, vjust = 0.1),
           axis.text.y  = element_text(size=size.text.y),
           legend.text=element_text(size=size.legend.text,vjust = -30),
           legend.title=element_text(size=size.legend.title,vjust = -30),
           legend.key.height=unit(unit.legend.h ,"line"),
           legend.key.width=unit(unit.legend.w,"line")) +
      scale_y_continuous(breaks=c(100,150,200,250,300,350,400,450,500),limits=c(80, 500)) +
	  scale_x_continuous(breaks=c(100,150,200,250,300,350,400,450,500),limits=c(80, 500)) +
	  labs(y = "Observed length (mm)") +
           labs(x = "Predicted length (mm)") 
           
           
           
results.par = m_grow3_zak_Linfcohort_kcohort.par # get model parameters 
#(model with cohort as predictor for k and linf)

pred = m_grow3_zak_Linfcohort_kcohort.rep #get predicted data

data.df = subset(allstreams.zakojska.df, Cohort!="P" & Cohort!="C11") #data

#create a dataframe with observed data (Obs) and predicted by model 
obpred = data.frame(Obs = data.df$Length, Pred = pred)

### compute Sum of Squared Errors
SSE.zak.Link.cohort = sum((obpred$Obs-obpred$Pred)^2)
### compute Total Sum of Squares
SST.zak.Link.cohort = sum((obpred$Obs-mean(obpred$Obs))^2)
#compute R squared
Rsq.zak.Link.cohort = 1-(SSE.zak.Link.cohort/SST.zak.Link.cohort)
#compute mean absolute error
mean.err.pred.zak.Link.cohort = sqrt(mean((obpred$Obs-obpred$Pred)^2))


#create plot with ggplot2         

    ggzak.mc2 <- ggplot(subset(obpred), aes(x = Pred, y = Obs)) 	  
ggzak.mc3  <- ggzak.mc2 + geom_point(pch = 16, size = 3, colour = "gray50") +
coord_fixed() +
geom_abline(intercept=0, slope=1, linetype = 2, lwd=1.2) +
 #      ggtitle("mediantimesnik") +
      theme(plot.title = element_text(lineheight=.8, face="bold"), 
      plot.background = element_blank()
   ,panel.grid.major = element_blank()
   ,panel.grid.minor = element_blank()
   ,panel.border = element_blank()
   ,panel.background = element_blank(),
   axis.line = element_line(color = 'black'),
   plot.margin = unit(c(1.5,1.5,1.5,1.5), "cm"),
   axis.title.x = element_text(size=size.label.x,vjust=-2),
           axis.text.x  = element_text(size=size.text.x ),
           axis.title.y = element_text(size=size.label.y, vjust = 0.1),
           axis.text.y  = element_text(size=size.text.y),
           legend.text=element_text(size=size.legend.text,vjust = -30),
           legend.title=element_text(size=size.legend.title,vjust = -30),
           legend.key.height=unit(unit.legend.h ,"line"),
           legend.key.width=unit(unit.legend.w,"line")) +
      scale_y_continuous(breaks=c(100,150,200,250,300,350,400,450,500),limits=c(80, 500)) +
	  scale_x_continuous(breaks=c(100,150,200,250,300,350,400,450,500),limits=c(80, 500)) +
	  labs(x = "Predicted length (mm)") +
	  labs(y = "")
           

           

### plot the 2 plots on 1 row, 2 columns
arrange_ggplot2(ggazak.mc1,ggzak.mc3,ncol = 2)


