source("ADMButils.s")


### this function gives Length at Age according to model parameters of fish
VBsnov3 = function (beta,k0,log_sigma_v,v,log_sigma_u,u,t0,log_sigma_w,w,plpo,maxage=15) {
    age = seq(1,maxage,1)
    Length_age = rep(0,length(age))
    gamma = exp(beta*10 + exp(log_sigma_v)*v)
    k = exp(k0) * exp( exp(log_sigma_u)*u )
    Linf = gamma 
    t0 = t0 + exp(log_sigma_w)*w
    
    for (ii in 1:length(age)) {
        
        Length_age[ii] = Linf*(1-exp(-k*(age[ii]-t0)))
        
    }
    if (plpo==0) {
        plot(Length_age ~ age, xlim = c(0,max(age)),ylim = c(0,max(Length_age)),pch=16)
    }
    
    else if (plpo==1) {
        points(Length_age ~ age,pch=15)
    }
    
    return(data.frame(Age=age,Length=Length_age))
}


NormCorr.f <- function(n=100, rho = 0.4,sdfirst = 1) { 
     firstvect = rnorm(n,mean = 0, sd = sdfirst)
     secondvect = rnorm(n) 
     Z = cbind(firstvect, rho*firstvect+sqrt(1-rho^2)*secondvect) 
    return(Z) 
} 


options(digits=2)

#########


maxage = 10

par(mfrow = c(2,5))

num.sim = 10

rho.vect = runif(num.sim, min = -1, max = 1)

set.seed(12)

for (i in 1:num.sim) {
#generate a mock file of 200 individuals of age 1 to 10 using 3 random effects uncorrelated
####values are similar to those found for the population of Zakojska

n.tot.ind = 400

beta = 0.58
beta_k0 = -1
beta_t0 = -0.38
log_sigma_v = -2
log_sigma_u = -2.3
log_sigma_w = -2.5

mockrand = data.frame(Mark = (1:n.tot.ind),u = rep(0,n.tot.ind),w = rep(0,n.tot.ind),v=rep(0,n.tot.ind))


uv = NormCorr.f(n.tot.ind,rho = rho.vect[i])
mockrand$u = uv[,1]
mockrand$v = uv[,2]
mockrand$w = rnorm(n.tot.ind,0,1)

uv.real.cor = cor.test(mockrand$u,mockrand$v)


mockdataset = data.frame(Mark = rep(mockrand$Mark,each=maxage),Length = rep(0,n.tot.ind*maxage),Age = rep(0,n.tot.ind*maxage))
contlen = 1
for (i in 1:n.tot.ind) {
plot.mock =VBsnov3(beta,beta_k0,log_sigma_v,mockrand[i,"v"],log_sigma_u,
mockrand[i,"u"],beta_t0, log_sigma_w,mockrand[i,"w"], 2,maxage=maxage)

mockdataset$Length[contlen: (maxage*i)] = plot.mock$Length
mockdataset$Age[contlen: (maxage*i)] = plot.mock$Age


contlen = length(mockdataset$Length[mockdataset$Length>0])+1
}




excluded = sample(1:nrow(mockdataset),(n.tot.ind*2))

mockdataset = mockdataset[-excluded,]


with(mockrand,cor.test(u,v))

#source("m_grow3_mock10.r")
source("m_grow3_mock3.r")

##### in the shell you should point to the current directory where also the m_grow3.tpl is stored####

#system("./m_grow10 -est")
system("./m_grow3 -est")

#results.par = par_read("m_grow10.par")

#results.par = par_read("m_grow10.par")

results.par = par_read("m_grow3.par")



uv.cor = cor.test(results.par$u,results.par$v)

plot(results.par$u,results.par$v,xlab= "u", ylab = "v",
pch = 16,col="gray40",xlim=c(-3,3),ylim=c(-3,3), cex.lab = 1.5)


text(-2,2.8,bquote(paste(italic(r)[R], " = ", .((uv.real.cor)$estimate))),cex=1.3)

text(-2,2,bquote(paste(italic(r)[E], " = ", .((uv.cor)$estimate))),cex=1.3)


}



## plot in RStudio with width of 1500, height 650
