source("ADMButils.s")


### this function gives Length at Age according to model parameters of fish
VBsnov3 = function (beta,k0,log_sigma_v,v,log_sigma_u,u,t0,log_sigma_w,w,plpo,maxage=15) {
    age = seq(1,maxage,1)
    Length_age = rep(0,length(age))
    gamma = exp(beta*10 + exp(log_sigma_v)*v)
    k = exp(k0) * exp( exp(log_sigma_u)*u )
    Linf = gamma 
    t0 = t0 + exp(log_sigma_w)*w
    
    for (ii in 1:length(age)) {
        
        Length_age[ii] = Linf*(1-exp(-k*(age[ii]-t0)))
        
    }
    if (plpo==0) {
        plot(Length_age ~ age, xlim = c(0,max(age)),ylim = c(0,max(Length_age)),pch=16)
    }
    
    else if (plpo==1) {
        points(Length_age ~ age,pch=15)
    }
    
    return(data.frame(Age=age,Length=Length_age))
}


NormCorr.f <- function(n=100, rho = 0.4,sdfirst = 1) { 
     firstvect = rnorm(n,mean = 0, sd = sdfirst)
     secondvect = rnorm(n) 
     Z = cbind(firstvect, rho*firstvect+sqrt(1-rho^2)*secondvect) 
    return(Z) 
} 


options(digits=2)

#########


maxage = 10

par(mfrow = c(2,5))

#num.sim = 30
num.sim = 1

n.tot.coh = 10  ## number of cohort including the first one

rho.vect = runif(num.sim, min = -1, max = 1)

#rho.vect.linfk = runif(num.sim, min = -1, max = 1)

rho.vect.linfk = 0.9

cor.test.coh = data.frame(coh.real = rep(0,num.sim), coh.est = rep(0,num.sim),
ind.real = rep(0,num.sim), ind.est = rep(0,num.sim))

predict.growth = data.frame(Age = rep(rep(1:maxage,n.tot.coh),num.sim),
Length = rep(rep(0,maxage*n.tot.coh),num.sim), 
Cohort = rep(rep(c(1:n.tot.coh),each = maxage),num.sim),
Sim = rep(1:num.sim,each = maxage*n.tot.coh)) 

contlen1 = 1

set.seed(21)

for (j in 1:num.sim) {
#generate a mock file of 200 individuals of age 1 to 10 using 3 random effects uncorrelated
####values are similar to those found for the population of Zakojska

n.tot.ind = 400


rand.cohort = sample(1:n.tot.coh, n.tot.ind, replace = T, 
prob = runif(n.tot.coh,min = 1, max = 5))

beta = 0.58
beta_k0 = -1
beta_t0 = -0.38
log_sigma_v = -3
log_sigma_u = -3
log_sigma_w = -5

mockrand = data.frame(Mark = (1:n.tot.ind),Cohort = rep(0,n.tot.ind), u = rep(0,n.tot.ind),w = rep(0,n.tot.ind),v=rep(0,n.tot.ind))
mock.coh = data.frame(linf = rep(0,n.tot.coh),k = rep(0,n.tot.coh)) 

uv = NormCorr.f(n.tot.ind,rho = rho.vect[j])
coh.linf.k = NormCorr.f(n.tot.coh,rho = rho.vect.linfk[j])
mock.coh$linf =  coh.linf.k[,1]/100
mock.coh$k =  coh.linf.k[,2]/10
mockrand$u = uv[,1]
mockrand$v = uv[,2]
mockrand$w = rnorm(n.tot.ind,0,1)

uv.real.cor = cor.test(mockrand$u,mockrand$v)


mockdataset = data.frame(Mark = rep(mockrand$Mark,each=maxage),Length = rep(0,n.tot.ind*maxage),Age = rep(0,n.tot.ind*maxage))
contlen = 1
for (i in 1:n.tot.ind) {

if (rand.cohort[i] == 1) {

Linf = beta
k = beta_k0

}

else {

Linf = beta + mock.coh$linf[rand.cohort[i]]
k = beta_k0 + mock.coh$k[rand.cohort[i]]
}


plot.mock =VBsnov3(Linf,k,log_sigma_v,mockrand[i,"v"],log_sigma_u,
mockrand[i,"u"],beta_t0, log_sigma_w,mockrand[i,"w"], 2,maxage=maxage)

mockdataset$Length[contlen: (maxage*i)] = plot.mock$Length
mockdataset$Age[contlen: (maxage*i)] = plot.mock$Age
mockdataset$Cohort[contlen: (maxage*i)] = rand.cohort[i]

contlen = length(mockdataset$Length[mockdataset$Length>0])+1
}


excluded = sample(1:nrow(mockdataset),(n.tot.ind*2))

mockdataset = mockdataset[-excluded,]


with(mockrand,cor.test(u,v))


source("m_grow3_mock.coh.r")

##### in the shell you should point to the current directory where also the m_grow3.tpl is stored####

#system("./m_grow3 -est")

system("./m_grow3")

#results.par = par_read("m_grow10.par")

#results.par = par_read("m_grow10.par")

results.par = par_read("m_grow3.par")

data = results.par


data$beta[2:length(data$beta)]	= data$beta[2:length(data$beta)] + data$beta[1]

data$beta = exp(data$beta*10)

data$beta_k0[2:length(data$beta_k0)]	= data$beta_k0[2:length(data$beta_k0)] + data$beta_k0[1]

data$beta_k0 = exp(data$beta_k0)

cor.test.coh$coh.est[j] = as.numeric(cor.test(data$beta,data$beta_k0)$estimate) 

mock.coh$linf[1] = beta
mock.coh$k[1] = beta_k0

data1 = mock.coh 
data1$linf[2:length(data1$linf)]	= data1$linf[2:length(data1$linf)] + data1$linf[1]

data1$linf = exp(data1$linf*10)

data1$k[2:length(data1$k)]	= data1$k[2:length(data1$k)] + data1$k[1]

data1$k = exp(data1$k)

cor.test.coh$coh.real[j] = as.numeric(cor.test(data1$linf,
data1$k)$estimate) 

uv.cor = cor.test(results.par$u,results.par$v)

cor.test.coh$ind.est[j] = uv.cor$estimate

cor.test.coh$ind.real[j] = uv.real.cor$estimate

plot(results.par$u,results.par$v,xlab= "u", ylab = "v",
pch = 16,col="gray40",xlim=c(-3,3),ylim=c(-3,3), cex.lab = 1.5)


text(-2,2.8,bquote(paste(italic(r)[R], " = ", .((uv.real.cor)$estimate))),cex=1.3)

text(-2,2,bquote(paste(italic(r)[E], " = ", .((uv.cor)$estimate))),cex=1.3)


for (yy in 1:n.tot.coh) {


data = results.par

if (yy == 1) {

Linf.coh = data$beta[1]
k.coh = data$beta_k0[1]

}

else {

Linf.coh = data$beta[1] + data$beta[yy]
k.coh = data$beta_k0[1] + data$beta_k0[yy]
}


plot.predict =VBsnov3(Linf.coh,k.coh,log_sigma_v,0,log_sigma_u,
0,beta_t0, log_sigma_w,0, 2,maxage=maxage)

contsim = sum(predict.growth$Sim<j)

predict.growth$Length[contlen1:((maxage*yy)+contsim)] = plot.predict$Length


contlen1 = length(predict.growth$Length[predict.growth$Length>0])+1

}






}



#cor.test.coh


#write.csv(cor.test.coh, file = "cor.test.coh.csv",row.names = FALSE)


predict.growth = data.frame(Age = rep(rep(1:maxage,n.tot.coh),num.sim),
Length = rep(rep(0,maxage*n.tot.coh),num.sim), 
Cohort = rep(rep(c(1:n.tot.coh),each = maxage),num.sim),
Sim = rep(1:num.sim,each = maxage*n.tot.coh)) 

contlen1 = 1

## plot in RStudio with width of 1500, height 650


gmock <- ggplot(subset(predict.growth,Sim == 14), aes(x = Age, y = Length)) 

#g <- ggplot(subset(uppidri.df,Mark!="NA" & Month == 9), aes(x = Age, y = Weight)) 

gmock1 <- gmock + geom_line(aes(group = Cohort),col="gray40",lwd = 1) +
      #geom_smooth(aes(group = 1), colour = 'red', size = 1) +
       ggtitle("Simulated data") +
      theme(plot.title = element_text(lineheight=.8, face="bold",size = 25),
    plot.background = element_blank()
   ,panel.grid.major = element_blank()
   ,panel.grid.minor = element_blank()
   ,panel.border = element_blank()
   ,panel.background = element_blank()) +theme(axis.line = element_line(color = 'black')) +
      scale_y_continuous(breaks=c(100,150,200,250,300,350,400),limits=c(80, 400)) +
	  scale_x_continuous(breaks=c(1,2,3,4,5,6,7,8),limits=c(0, 8)) +
	   theme(plot.margin = unit(c(2,1,2,1), "cm"),
	   axis.title.x = element_text(size=20,vjust=-2),
           axis.text.x  = element_text(size=16),
           axis.title.y = element_text(size=20, vjust = 0.1),
           axis.text.y  = element_text(size=16))
	  
	  gmock1 
