source("ADMButils.s")
library(dplyr)

### this function gives Length at Age according to model parameters of fish
VBsnov3 = function (beta,k0,log_sigma_v,v,log_sigma_u,u,t0,log_sigma_w,w,plpo,maxage=15) {
    age = seq(1,maxage,1)
    Length_age = rep(0,length(age))
    gamma = exp(beta*10 + exp(log_sigma_v)*v)
    k = exp(k0) * exp( exp(log_sigma_u)*u )
    Linf = gamma 
    t0 = t0 + exp(log_sigma_w)*w
    
    for (ii in 1:length(age)) {
        
        Length_age[ii] = Linf*(1-exp(-k*(age[ii]-t0)))
        
    }
    if (plpo==0) {
        plot(Length_age ~ age, xlim = c(0,max(age)),ylim = c(0,max(Length_age)),pch=16)
    }
    
    else if (plpo==1) {
        points(Length_age ~ age,pch=15)
    }
    
    return(data.frame(Age=age,Length=Length_age))
}


NormCorr.f <- function(n=100, rho = 0.4,sdfirst = 1) { 
     firstvect = rnorm(n,mean = 0, sd = sdfirst)
     secondvect = rnorm(n) 
     Z = cbind(firstvect, rho*firstvect+sqrt(1-rho^2)*secondvect) 
    return(Z) 
} 


options(digits=2)

#########

set.seed(25)


maxage = 10

par(mfrow = c(2,5))

num.sim = 1

rho.vect = runif(num.sim, min = -1, max = 1)



for (i in 1:num.sim) {
#generate a mock file of 200 individuals of age 1 to 10 using 3 random effects uncorrelated
####values are similar to those found for the population of Zakojska

n.tot.ind = 400

beta = 0.58
beta_k0 = -1
beta_t0 = -0.38
log_sigma_v = -2
log_sigma_u = -2.3
log_sigma_w = -2.5

Linf.vb = exp(beta*10)
k.vb = exp(beta_k0)

mockrand = data.frame(Mark = (1:n.tot.ind),u = rep(0,n.tot.ind),w = rep(0,n.tot.ind),v=rep(0,n.tot.ind))


uv = NormCorr.f(n.tot.ind,rho = rho.vect[i])
mockrand$u = uv[,1]
mockrand$v = uv[,2]
mockrand$w = rnorm(n.tot.ind,0,1)

uv.real.cor = cor.test(mockrand$u,mockrand$v)


mockdataset = data.frame(Mark = rep(mockrand$Mark,each=maxage),
Length = rep(0,n.tot.ind*maxage),Age = rep(0,n.tot.ind*maxage), 
Linf = rep(0,each=maxage), k = rep(0,each=maxage) )
contlen = 1
for (i in 1:n.tot.ind) {
plot.mock =VBsnov3(beta,beta_k0,log_sigma_v,mockrand[i,"v"],log_sigma_u,
mockrand[i,"u"],beta_t0, log_sigma_w,mockrand[i,"w"], 2,maxage=maxage)

mockdataset$Length[contlen: (maxage*i)] = plot.mock$Length
mockdataset$Age[contlen: (maxage*i)] = plot.mock$Age
mockdataset$Linf[contlen: (maxage*i)] = exp(beta*10 + exp(log_sigma_v)*mockrand[i,"v"])
mockdataset$k[contlen: (maxage*i)] = exp(beta_k0) * exp( exp(log_sigma_u)*mockrand[i,"u"] )


contlen = length(mockdataset$Length[mockdataset$Length>0])+1
}




### linear size-dependent mortality

### estimate the slope

max_L_1 = max(subset(mockdataset, Age == 1)$Length)
min_L_1 = min(subset(mockdataset, Age == 1)$Length)

 # say that for min length the probability of dying is 0.7, for max is 0.7
 max_mort = 0.7
 min_mort = 0.1
 
 slope_mort = (min_mort - max_mort )/(max_L_1 - min_L_1)
 
mort.df =  mockdataset[mockdataset$Age == 1,c("Mark","Length")]
mort.df$mort = 0
mort.df$rand = runif(nrow(mort.df),min = 0, max = 1) 
mort.df$pot.mort = 0
mort.df$mort.realiz = 0

mort.f = function(min_l,max_mort,slope, fish_l) {

mort.pot = max_mort  + slope_mort*(fish_l - min_l)

return(mort.pot)

}

for (mm in 1:nrow(mort.df)) {

mort.df$pot.mort[mm] = mort.f(min_L_1, max_mort,slope_mort, mort.df$Length[mm])

} 

mort.df$mort.realiz = with(mort.df, ifelse(rand >pot.mort,0,1))

fish_die = mort.df$Mark[mort.df$mort.realiz == 1]


mockdataset_mort = mockdataset[-which(mockdataset$Mark %in% fish_die & mockdataset$Age>1),]


die_unique = mockdataset[which(mockdataset$Mark %in% fish_die & mockdataset$Age==1), "Mark"]


#excluded = sample(1:nrow(mockdataset),(n.tot.ind*2))

#mockdataset = mockdataset[-excluded,]


data_mockdataset = mockdataset



with(mockrand,cor.test(u,v))

#source("m_grow3_mock10.r")


source("m_grow3_mock3_size_mort.r")

##### in the shell you should point to the current directory where also the m_grow3.tpl is stored####

#system("./m_grow10 -est")
system("./m_grow3 -est")

#results.par = par_read("m_grow10.par")

#results.par = par_read("m_grow10.par")

results.par.no.mort = par_read("m_grow3.par")

uv.cor.no.mort = cor.test(results.par.no.mort$u,results.par.no.mort$v)

data_mockdataset = mockdataset_mort

source("m_grow3_mock3_size_mort.r")

##### in the shell you should point to the current directory where also the m_grow3.tpl is stored####

#system("./m_grow10 -est")
system("./m_grow3 -est")


results.par.mort = par_read("m_grow3.par")

uv.cor.mort = cor.test(results.par.no.mort$u,results.par.no.mort$v)


k_mort = exp(results.par.mort$beta_k0) * 
exp( exp(results.par.mort$log_sigma_u)*results.par.mort$u ) 
## compute k (Eq. 7 in the ms)

Linf_mort = exp(results.par.mort$beta*10 + 
exp(results.par.mort$log_sigma_v)*results.par.mort$v)


k_no.mort = exp(results.par.no.mort$beta_k0) * 
exp( exp(results.par.no.mort$log_sigma_u)*results.par.no.mort$u ) 
## compute k (Eq. 7 in the ms)

Linf_no.mort = exp(results.par.no.mort$beta*10 + 
exp(results.par.no.mort$log_sigma_v)*results.par.no.mort$v)

### Plot an histogram of differences between Linf and estimated estimated with and without
# size-selective mortality (results are in mm)

par(mfrow = c(1,2))
hist(Linf_mort[die_unique] - Linf_no.mort[die_unique], xlab = "Difference in Linf (mm)")
hist(k_mort[die_unique] - k_no.mort[die_unique],xlab = "Difference in k (1/y)")


### Estimates at the population level

Linf.pop.mort = exp(results.par.mort$beta*10)
k.pop.mort = exp(results.par.mort$beta_k0)

Linf.pop.no.mort = exp(results.par.no.mort$beta*10)
k.pop.no.mort = exp(results.par.no.mort$beta_k0)


}



## plot in RStudio with width of 1500, height 650
