DATA_SECTION

  init_int n				// Number of data points
  init_vector length(1,n)		// Length of fish
  init_vector age(1,n)			// Age of fish
  init_int total_marks			// Number of unique individuals
  init_ivector start(1,total_marks)	// Start row in data frame for j'th individual
  init_ivector stop(1,total_marks)	// Stop row in data frame for j'th individual
  init_int q				// Number of columns in design matrix
  init_matrix  X(1,n,1,q)		// Design matrix
  init_int q_k0				//   --||--
  init_matrix X_k0(1,n,1,q_k0)		//   --||--
  init_int q_t0				//   --||--
  init_matrix  X_t0(1,n,1,q_t0)		//   --||--
  init_int check

  !! cout << "check=" << check << endl;                        
  PARAMETER_SECTION

  init_bounded_vector beta(1,q,-1,1,1)	// Regression parameters
  init_bounded_vector beta_k0(1,q_k0,-5.0,2.0,1)	// -5 is the lower bound
  init_bounded_vector beta_t0(1,q_t0,-5,1,1)	// -5 is the lower bound
  init_bounded_number psi(-0.0,1.0,-1)
  init_bounded_number log_sigma_u(-5.0,2.0,2)	// log(individual sd) for k  #if - 2 == parameter fixed and then needs to be fixed the value in the pin file
  init_bounded_number log_sigma_v(-5.0,2.0,2)	// log(individual sd) for gamma
  init_bounded_number log_sigma_w(-5.0,2.0,-2)	// log(individual sd) for t0  // -2 if we want to block it, 2 if we want to estimate,valid for all fixed effects
  init_bounded_number log_sigma(0.0,7.0,1)	// log(measurement error sd)
  random_effects_vector u(1,total_marks,2)	// Unscaled random effects for k
  random_effects_vector v(1,total_marks,2)	// Unscaled random effects for gamma
  random_effects_vector w(1,total_marks,-2)	// Unscaled random effects for t0 // -2 if we want to block it, 2 if we want to estimate,valid for all random effects. If we shut down the fixed, we also have to shut down the random
  
  sdreport_vector Linf_sd(1,q)	
  sdreport_vector k_sd(1,q_k0)

  objective_function_value g

PROCEDURE_SECTION

  int i;
  for(i=1;i<=total_marks;i++)
  {
    fit_individual(i,beta,beta_k0,beta_t0,psi,log_sigma,log_sigma_u,log_sigma_v,log_sigma_w,u(i),v(i),w(i));
  }
  
   if(sd_phase())
  {
    for(i=1;i<=q;i++)
    {
    	if(i==1) {
      	Linf_sd(i) = exp(beta(i)*10.0);
    	}
    
    	else {
    	Linf_sd(i) = exp(beta(i)*10.0) + exp(beta(1)*10.0);
    	}
    }
    
    for(i=1;i<=q_k0;i++)
    {
    	if(i==1){
     	k_sd(i) =  exp(beta_k0(i));
  		}
  		
  		else {
		k_sd(i) =  exp(beta_k0(i)) + exp(beta_k0(1));
		}
  		
	}

	}


SEPARABLE_FUNCTION void fit_individual(int i,const dvar_vector& beta, const dvar_vector& beta_k0, const dvar_vector& beta_t0, const prevariable& psi, const prevariable& log_sigma, const prevariable& log_sigma_u, const prevariable& log_sigma_v, const prevariable& log_sigma_w, const prevariable& u,const prevariable& v,const prevariable& w)

  dvariable sigma = exp(log_sigma);  
  dvariable U = exp(log_sigma_u)*u;
  dvariable V = exp(log_sigma_v)*v;
  dvariable W = exp(log_sigma_w)*w;
  
  dvariable k0 = mfexp(X_k0(start(i))*beta_k0); 
  dvariable k = k0*exp(U);

  dvariable t0 = X_t0(start(i))*beta_t0 + W; 
  dvariable gamma = mfexp(X(start(i))*beta*10.0 + V);	// The factor 10 is to scale beta better

  dvariable k_psi1 = 1.0;			// When psi=1
  
  g -= -0.9189385 - 0.5*square(u);				// N(0,1) Gaussian random effect
  g -= -0.9189385 - 0.5*square(v);				// N(0,1) Gaussian random effect
  g -= -0.9189385 - 0.5*square(w);				// N(0,1) Gaussian random effect

  int j;
  for(j=start(i);j<=stop(i);j++)
  {    
    dvariable Linf = gamma*k_psi1;
    dvariable L = Linf*(1.0-exp(-k*(age(j)-t0)));
    g -= -0.9189385 -log_sigma - 0.5*square((length(j)-L)/sigma);
  }	

REPORT_SECTION

  int i;
  for(i=1;i<=total_marks;i++)
  {

  dvariable sigma = exp(log_sigma);  
  dvariable U = exp(log_sigma_u)*u(i);
  dvariable V = exp(log_sigma_v)*v(i);
  dvariable W = exp(log_sigma_w)*w(i);
  
  dvariable k0 = mfexp(X_k0(start(i))*beta_k0); 
  dvariable k = k0*exp(U);

  dvariable t0 = X_t0(start(i))*beta_t0 + W; 
  dvariable gamma = mfexp(X(start(i))*beta*10.0 + V);	// The factor 10 is to scale beta better

  dvariable k_psi1 = 1.0;			// When psi=1

  int j;
  for(j=start(i);j<=stop(i);j++)
  {    
    dvariable Linf = gamma*k_psi1;
    dvariable L = Linf*(1.0-exp(-k*(age(j)-t0)));
    report << L << endl;
  }	
  }  
  
TOP_OF_MAIN_SECTION
  arrmblsize = 40000000L;
  gradient_structure::set_GRADSTACK_BUFFER_SIZE(3000000);
  gradient_structure::set_CMPDIF_BUFFER_SIZE(200000);
  gradient_structure::set_MAX_NVAR_OFFSET(460404);

